#include "filed.h"
#include "queue.h"
#include "main.h"
#include "socket.h"
#include "externs.h"
#include "session.h"
#include "log.h"
#include "parse.h"
#include "flags.h"
#include "io.h"
#include "sys.h"
#include "dbq.h"
#include "peer.h"
#include "url.h"

#include <string.h>
#include <stdio.h>
#include <sys/socket.h>

void process_io(void) {
	struct connection *c;
	struct connection *d;

	c = filed->con_head;

	while(c) {
		if(c->flags & CFLAG_NEW) {
			if(check_access(c->remote_addr))
				c_send(c,generate_welcome());
				else {
					c_send(c,"ERR access is denied for %s",c->remote_addr);
					c_hangup(c);
				}
					
			c->flags -= CFLAG_NEW;
		}
		d = c->next;
		if(c->in) process_input(c);
		if(c->err) process_errors(c);
		c = d;
	}
	
}

char *generate_welcome(void) {
	static char outbuf[1024];
	sprintf(outbuf,"OK %s",translate_string(get_data(filed->cf,"welcome-banner")));
	return(outbuf);
}

void process_errors(struct connection *c) {
	struct queue *q;
	q = c->err;

	if((q && (q->flags & QFLAG_HANGUP)) && !(pending_tx(c)))
		process_disconnect(c); else
	if((q && (q->flags & QFLAG_DISCONNECT)) && !(pending_rx(c))) 
		process_disconnect(c);
}

void process_input(struct connection *c) {
	/* check to see if the current queue entry is a disconnect */
	struct queue *q;
	q = c->in;
	

	switch(c->type) {
		case CTYPE_HUMAN:
			process_human(c);
			break;
		case CTYPE_JAVA:
			process_java(c);
			break;
		case CTYPE_PEER:
			process_peer(c);
			break;
	}
}

void process_human(struct connection *c) {
	struct queue *q;
	int i=0;

	while((q = c->in) && i++ < 3) {
		if(!c->in->offset) {
			parse_human(c,q);
			c->in = rm_queue(c->in,q);
		} else break;
	}
}

void process_java(struct connection *c) {
	struct queue *q;
	int i=0;

	while((q = c->in) && i++ < 3) {
		if(!c->in->offset) {
			parse_java(c,q);
			c->in = rm_queue(c->in,q);
		} else break;
	}
}

void process_peer(struct connection *c) {
	struct queue *q;
	int i=0;

	while((q = c->in) && i++ < 3) {
		if(!c->in->offset) {
			parse_peer(c,q);
			c->in = rm_queue(c->in,q);
		} else break;
	}
}

void process_disconnect(struct connection *c) {
	
	dlog(DEBUG_CODE,"process_disconnect() called");

	shutdown(c->fd,2);
	close(c->fd);
	remove_connection(c);
	free_connection(c);
}

void parse_human(struct connection *c, struct queue *q) {

	struct parse *p;
	char *cmd;

	dlog(DEBUG_IO,"parse_human() called by %d: %s",c->id,q->data);

	p = parse_line(q->data);
	
	cmd = get_token_s(p,0);
	
		if(!strcasecmp(cmd,"bye")) {
			c_send(c,"BYE");
			c_hangup(c);
		} else if(!strcasecmp(cmd,"test")) {
			c_send(c,"OK test initiated");
			dbq(1,c->id,"test");
		} else if(!strcasecmp(cmd,"shmem")) {
			shmem_subparse(c,p);
		} else if(!strcasecmp(cmd,"peer")) {
			peer_subparse(c,p);
		} else if(!strcasecmp(cmd,"down")) {
			go_down(c); /* what kerianne never did, no matter what those fuckheads at fuckedcompany said */
		} else if(!strcasecmp(cmd,"up")) {
			go_up(c);
		} else if(!strcasecmp(cmd,"who")) {
			do_who(c);
		} else if(!strcasecmp(cmd,"debug")) {
			debug_subparse(c,p);	
		} else if(!strcasecmp(cmd,"shutdown")) {
			do_shutdown(c,p);
		} else c_send(c,"ERR I don't know the command [%s]. Try help.",cmd);	
	free_parse(p);
}

void do_shutdown(struct connection *c, struct parse *p) {
	char *ch;
	char *msg;

	ch = get_token_s(p,1);

	if(ch && !strcasecmp(ch,"force")) {
		elog("Admin %s performed a forced shutdown",c->name);
		die();
	}
	
	/* todo: all, message */
}

void do_who(struct connection *c) {
	struct connection *d = filed->con_head;

	while(d) {
		c_send(c,"[%d] %s %s: %s %s",d->id,c_str_type(d->type),d->remote_addr,d->name,c_str_flags(d->flags));
		d = d->next;
	}
	return;
}

void shmem_subparse(struct connection *c, struct parse *p) {
	char *cmd = get_token_s(p,1);

	if(!cmd) {
		c_send(c,"ERR need a subcommand for shmem");
	} else if(!strcasecmp(cmd,"dump")) {
		shmem_connection_dump(c);
	} else if(!strcasecmp(cmd,"show")) {
		shmem_slot_show(c,p);
	} else c_send(c,"ERR no subcommand [%s] for command shmem");
	return;
}

void peer_subparse(struct connection *c, struct parse *p) {
	char *cmd = get_token_s(p,1);
		
	if(!cmd) {
		c_send(c,"ERR need a subcommand for peer");
	} else if(!strcasecmp(cmd,"dump")) {
		peer_connection_dump(c,p);
	} else c_send(c,"ERR no subcommand [%s] for command peer");
}

void debug_subparse(struct connection *c, struct parse *p) {
	char *cmd = get_token_s(p,1);
	int ret;
	
	if(!cmd) {
		c_send(c,"ERR need a subcommand for debug");
	} else if(!strcasecmp(cmd,"trace")) {
		ret = build_debug_flags(get_token(p,2));
		if(!ret) {
			c_send(c,"ERR no items to trace found. (use notrace to turn off tracing");
		} else {
			if(ret & DEBUG_CODE) {
				ret -= DEBUG_CODE;
				c_send(c,"NOTE: code option was stripped to prevent looping");
			}

			if(!(sys_flags & FLAG_DEBUGGING)) sys_flags += FLAG_DEBUGGING;
			c->debug_mask = ret;
			c_send(c,"OK now debugging: %s",get_debug_string(c->debug_mask));
		}
	} else if(!strcasecmp(cmd,"notrace")) {
		c->debug_mask = 0;
		c_send(c,"OK not tracing");
	} else if(!strcasecmp(cmd,"status")) {
		c_send(c,"OK now debugging: %s",get_debug_string(c->debug_mask));
	} else c_send(c,"ERR no subcommand [%s] for command debug",cmd);
}


	

void parse_java(struct connection *c, struct queue *q) {
	struct parse *p;
	char *cmd;
	char *arg1;

	dlog(DEBUG_IO,"parse_java() called by %d: %s",c->id,q->data);
	p = parse_line(q->data);

	cmd = get_token_s(p,0);
	arg1 = get_token_s(p,1);

	if(!strcasecmp(cmd,"A") || !strcasecmp(cmd,"S")) {
		process_link(c,p);
	} else if(!strcasecmp(cmd,"clear")) {
		do_clear(c,p);
	} else if(!strcasecmp(cmd,"admin")) {
		c_send(c,"OK human interface selected");
		c->type = CTYPE_HUMAN;
		if(arg1) strcpy(c->name,arg1);
	} else if(!strcasecmp(cmd,"peer")) {
		strcpy(c->name,"");
		peer_process_peer(c,p);
	} else if(!strcasecmp(cmd,"bye")) {
		c_send(c,"BYE");
		c_hangup(c);
	} else c_send(c,"ERR ULINK \"unknown command [%s]\"",cmd);

	
	free_parse(p);
	
}

void parse_peer(struct connection *c, struct queue *q) {
	struct parse *p;
	char *cmd;
	char *arg1;
	
	p = parse_line(q->data);

	cmd = get_token_s(p,0);
	arg1 = get_token_s(p,1);

	dlog(DEBUG_IO,"parse_peer() called by %d: %s",c->id,q->data);
	if(!cmd) {
		c_send(c,"ERR \"No command\"");
		return;
	}

	if(!strcasecmp(cmd,"OK")) {
		peer_process_ok(c,p);
	} else if(!strcasecmp(cmd,"ERR")) {
		peer_process_err(c,p);
	} else if(!strcasecmp(cmd,"PEER")) {
		peer_process_peer(c,p);
	} else if(!strcasecmp(cmd,"SX")) {
		url_process_sx(c,p);
	} else if(!strcasecmp(cmd,"TX")) {
		peer_process_tx(c,p);
	} else if(!strcasecmp(cmd,"HB")) {
		peer_process_hb(c,p);
	} else if(!strcasecmp(cmd,"bye")) {
		c_send(c,"BYE");
		c_hangup(c);
	} else c_send(c,"ERR ULINK \"unknown command [%s]\"",cmd);

}

void do_clear(struct connection *c, struct parse *p) {
	char *sid = get_token_s(p,1);
	struct session *s;

	if(!sid) {
		c_send(c,"ERR \"Clear requres an argument\"");
		return;
	}
	
	s = get_session_i(sid);
	if(!s) {
		c_send(c,"ERR \"Session [%s] does not exist\"",sid);
		return;
	}

	shmem_session_kill(s->session_seq);
	c_send(c,"OK [%s] CLEARED",sid);
}

void go_down(struct connection *c) {
	if(!(filed->peer_flags & PFLAG_DOWN)) filed->peer_flags += PFLAG_DOWN;
	if(!(filed->peer_flags & PFLAG_ADMINDOWN)) filed->peer_flags += PFLAG_ADMINDOWN;
	c_send(c,"OK we are now DOWN");
}

void go_up(struct connection *c) {
	if((filed->peer_flags & PFLAG_DOWN)) filed->peer_flags -= PFLAG_DOWN;
	if((filed->peer_flags & PFLAG_ADMINDOWN)) filed->peer_flags -= PFLAG_ADMINDOWN;
	c_send(c,"OK we are now UP");
}

