#ifndef __shmem_h
#define __shmem_h
#include "filed.h"
#include "parse.h"

/* master (exportable) functions */

void bind_shmem(void);
void bind_shmem_client(void);
void handle_shmem(void);
void wait_for_db(void);
void wait_for_main(void);
void display_shmem(void);
void shmem_db_ready(void);

int shmem_ping_db_thread(void);
void shmem_connection_dump(struct connection *c);
void shmem_slot_show(struct connection *c, struct parse *p);
char *shmem_msg_flags_string(int flags);

void shmem_respond_to_filed_pings(void);
int shmem_check(void);
void shmem_shutdown(void);
void shmem_handle(void);
void shmem_do_delete(void);
void shmem_ins_delete(int i);

/* session functions */

void shmem_file_add(char *from, char *to, int session_id);
void shmem_session_kill(int skey);
void shmem_file_remove(int slot);

/* hack functions */

struct db *shmem_db_get_done(void);
void shmem_db_ins(struct db *db);
int shmem_db_id_present(int id);
int shmem_db_free(void);
char *shmem_file_flags_string(int flags);

/* server thread functions */
struct db *shmem_db_get_next(void);
void shmem_db_set(struct db *db);



#define PERMS 0666
#define SHMEM_NUM_QUEUES 32
#define SHMEM_DATA_SIZE 1024

#define SHMEM_MAX_FILES 10000
 
struct msg {
	/* master */

	int master_flags;	
	int max_entry;

	/* files */

	char from[SHMEM_MAX_FILES][128];
	char to[SHMEM_MAX_FILES][128];
	short flags[SHMEM_MAX_FILES];
	int id[SHMEM_MAX_FILES];

	/* database */

	char message[SHMEM_NUM_QUEUES][1024];
	short message_flags[SHMEM_NUM_QUEUES];
	int connection[SHMEM_NUM_QUEUES];
	int filter[SHMEM_NUM_QUEUES];

	/* note - these are only used to talk to the TCP server */
	/* all database refereshes of the shmem segment come directly from the database
	   server */

};

struct todel {
	int slot;
	time_t delete_at;
	struct todel *next;
	struct todel *prev;
};

/* master flags */

#define SHMEM_FLAG_READY 1
#define SHMEM_FLAG_DBREADY 2
#define SHMEM_FLAG_WAIT 4 /* tells samba to wait because filed is busy */
#define SHMEM_FLAG_DBPING 8 /* filed sets this when it suspects the db
			process of dying. The db unsets it. filed
			_will_ fork a new process if a ping fails for 1 sec
			and reissue all queues */
#define SHMEM_FLAG_PING 16

/* message flags */

#define SHMEM_CLAIMED 1
#define SHMEM_PROCESSED 2
#define SHMEM_OKTOGO 4
#define SHMEM_MORECOMING 8
#define SHMEM_DONE 16
#define SHMEM_ERR 32


/* file flags */

#define SHMEM_ACTIVE 1
#define SHMEM_OPENED 2
#define SHMEM_TOUCHED 4
#define SHMEM_STATIOND 8
#define SHMEM_INS_DEL 16

#endif
